import 'core.dart';

class M4eExceptionCode {
  final String _exceptionCode;

  const M4eExceptionCode._(this._exceptionCode);

  static const kNoAuthUserExceptionCode = M4eExceptionCode._('NO_AUTH_USER');
  static const kNoAuthUserCredentialExceptionCode =
      M4eExceptionCode._('NO_CACHED_AUTH_CREDENTIAL');
  static const kNoCacheDataExceptionCode = M4eExceptionCode._('NO_CACHE_DATA');
  static const kFailedTransactionExceptionCode =
      M4eExceptionCode._('FAILED_TRANSACTION');
  static const kUndefinedExceptionCode = M4eExceptionCode._('UNDEFINED');
  static const kConnectionTimeoutExceptionCode =
      M4eExceptionCode._('CONNECTION_TIMEOUT');
  static const kNoInternetConnectionExceptionCode =
      M4eExceptionCode._('NO_INTERNET_CONNECTION');

  static M4eExceptionCode parse(String type) {
    return M4eExceptionCode._(type);
  }

  @override
  String toString() => _exceptionCode;

  bool operator ==(o) =>
      (o is M4eExceptionCode) && o._exceptionCode == this._exceptionCode;

  @override
  int get hashCode => _exceptionCode.hashCode;
}

class M4eExceptionMessages {
  static final kNoAuthUser = M4eAuthException(
    code: M4eExceptionCode.kNoAuthUserExceptionCode,
    message: 'No authenticated m4e user account',
  );

  static final kNoAuthCredential = M4eAuthException(
    code: M4eExceptionCode.kNoAuthUserCredentialExceptionCode,
    message: 'No user authentication credential found',
  );

  static final kNoCacheDataException = M4eCacheException(
    code: M4eExceptionCode.kNoCacheDataExceptionCode,
    message: 'No cache data!',
  );

  static final kFailedTransactionException = M4eServerException(
    code: M4eExceptionCode.kFailedTransactionExceptionCode,
    message: 'Transaction failed to complete!',
  );

  static final kUndefinedException = M4eServerException(
    code: M4eExceptionCode.kUndefinedExceptionCode,
    message: 'Something went wrong!',
  );

  static final kConnectionTimeOutException = M4eServerException(
    code: M4eExceptionCode.kConnectionTimeoutExceptionCode,
    message: 'Connection timeout!',
  );

  static final kNoInternetConnectionException = M4eServerException(
    code: M4eExceptionCode.kNoInternetConnectionExceptionCode,
    message: 'No internet connection!',
  );
}

class M4eClientRequest {
  /// connection timeout
  static const kConnectionTimeout = 60000;
  static const kRecieveTimeout = 60000;
}

class M4ePersistence {
  static const kM4eAuthToken = 'M4E_AUTH_TOKEN';
  static const kM4eAuthCredential = 'M4E_AUTH_CREDENTIAL';
  static const kM4eUserWallets = 'M4E_USER_WALLETS';
  static const kM4eUserTransactionLogs = 'M4E_USER_TRANSACTION_LOGS';
}

class M4eCurrency {
  static const kGHS = 'GHS';
}

class EntityWalletPermissions {
  final String _permission;

  const EntityWalletPermissions._(this._permission);

  factory EntityWalletPermissions.owner() => EntityWalletPermissions._(
      'access=SEE&access=VIEW&access=PAY&access=ALTER&access=GRANT&access=CREATE&access=DESTROY');
  factory EntityWalletPermissions.payTo() =>
      EntityWalletPermissions._('access=PAY_TO&access=SEE');

  static EntityWalletPermissions parse(String type) {
    return EntityWalletPermissions._(type);
  }

  @override
  String toString() => _permission;

  @override
  bool operator ==(Object o) {
    if (identical(this, o)) return true;

    return o is EntityWalletPermissions && o._permission == _permission;
  }

  @override
  int get hashCode => _permission.hashCode;
}
