import 'package:uuid_type/uuid_type.dart';

class M4eUniqueId {
  final Uuid value;

  /// Generates a version-4 unique uid
  factory M4eUniqueId() {
    return M4eUniqueId._(Uuid(uuid.v4()));
  }

  /// Converts a unique uid string to unique uid type
  factory M4eUniqueId.fromUniqueString(String uniqueId) {
    assert(uniqueId != null, 'M4eUniqueId cannot be null');
    return M4eUniqueId._(Uuid(uniqueId));
  }

  /// Serializes a unique uid string from a `Map` object with the below structure to unique uid type
  ///
  /// Example structure:
  ///
  /// `
  /// {
  ///   'id': 'f9eb93fe-07d9-4b38-8117-f709aa438a18'
  /// }
  /// `
  ///
  factory M4eUniqueId.fromJson(Map<String, dynamic> json) {
    assert(json != null && json.isNotEmpty,
        'M4eUniqueId.fromJson cannot be null or empty');
    return M4eUniqueId._(Uuid(json['id']));
  }

  const M4eUniqueId._(this.value);

  @override
  bool operator ==(Object o) {
    if (identical(this, o)) return true;

    return o is M4eUniqueId && o.value == value;
  }

  @override
  int get hashCode => value.hashCode;

  @override
  String toString() => '${value.toString()})';
}
