import 'package:equatable/equatable.dart';
import 'package:flutter/foundation.dart';

class ConfigFlavour {
  final String _type;

  const ConfigFlavour._(this._type);

  static const production = ConfigFlavour._('PRODUCTION');
  static const staging = ConfigFlavour._('STAGING');
  static const development = ConfigFlavour._('DEVELOPMENT');

  static ConfigFlavour parse(String type) {
    return ConfigFlavour._(type);
  }

  @override
  operator ==(cmp) => (cmp is ConfigFlavour) && cmp._type == _type;

  @override
  int get hashCode => _type.hashCode;

  @override
  String toString() => _type;
}

class M4eSDKConfig extends Equatable {
  final String appId;
  final ConfigFlavour flavour;
  final String baseUrl;

  const M4eSDKConfig._(
    this.appId,
    this.flavour,
    this.baseUrl,
  );

  // TODO: implement other sdk-config envs.
  // TODO: update base-urls to m4e server
  static const _developmentBaseUrl = 'https://bascule-staging.herokuapp.com';
  static const _stagingBaseUrl = 'https://bascule-staging.herokuapp.com';

  /// Provdes development environment config to the m4e-sdk
  factory M4eSDKConfig.development({@required String appId}) {
    assert(appId != null, 'M4eSDKConfig [appId] should not be null');
    return M4eSDKConfig._(
        appId, ConfigFlavour.development, _developmentBaseUrl);
  }

  /// Provdes staging environment config to the m4e-sdk
  factory M4eSDKConfig.staging({@required String appId}) {
    assert(appId != null, 'M4eSDKConfig [appId] should not be null');
    return M4eSDKConfig._(appId, ConfigFlavour.development, _stagingBaseUrl);
  }

  @override
  List<Object> get props => [appId, flavour, baseUrl];

  @override
  String toString() =>
      'M4eSDKConfig(appId: $appId, flavour: $flavour, baseUrl: $baseUrl)';
}
