import 'package:equatable/equatable.dart';
import 'package:flutter/foundation.dart';

import '../../core/core.dart';
import '../../models/models.dart';

/// Form parameters required for m4e-sdk
/// authentication phone number verification
///
class M4eAuthPhoneNumberForm extends Equatable {
  final M4eUniqueId id;
  final M4ePhoneNumber phoneNumber;

  const M4eAuthPhoneNumberForm._(this.id, this.phoneNumber);

  factory M4eAuthPhoneNumberForm(
      {@required M4eUniqueId id, @required M4ePhoneNumber phoneNumber}) {
    assert(id != null, 'M4eAuthPhoneNumberForm [id] should not be null');
    assert(phoneNumber != null,
        'M4eAuthPhoneNumberForm [phoneNumber] should not be null');

    return M4eAuthPhoneNumberForm._(id, phoneNumber);
  }

  M4eAuthPhoneNumberForm copyWith(
      {M4eUniqueId id, M4ePhoneNumber phoneNumber}) {
    return M4eAuthPhoneNumberForm._(
      id ?? this.id,
      phoneNumber ?? this.phoneNumber,
    );
  }

  factory M4eAuthPhoneNumberForm.fromJson(Map<String, dynamic> json) {
    return M4eAuthPhoneNumberForm(
      id: M4eUniqueId.fromUniqueString(json['id']),
      phoneNumber: M4ePhoneNumber.fromJson(json['phone']),
    );
  }

  Map<String, dynamic> toJson() {
    return {
      'id': id.value.toString(),
      'phone': phoneNumber.toJson(),
    };
  }

  @override
  List<Object> get props => [id, phoneNumber];

  @override
  String toString() =>
      'M4eAuthPhoneNumberForm(id: ${id.value.toString()}, phoneNumber: ${phoneNumber.number}, country: ${phoneNumber.country})';
}
