import 'package:equatable/equatable.dart';
import 'package:flutter/foundation.dart';

import '../../core/core.dart';
import '../../models/models.dart';

enum M4eTransactionStatus { pending, failed, success }

class M4eTransactionMeta extends Equatable {
  final String callbackUrl;
  final String purpose;
  final String data;

  const M4eTransactionMeta._(this.callbackUrl, this.purpose, this.data);

  factory M4eTransactionMeta({
    @required String callbackUrl,
    @required String purpose,
    @required String data,
  }) {
    assert(callbackUrl != null,
        'M4eTransactionMeta [callbackUrl] should not be null');
    assert(purpose != null, 'M4eTransactionMeta [purpose] should not be null');
    assert(data != null, 'M4eTransactionMeta [data] should not be null');
    return M4eTransactionMeta._(callbackUrl, purpose, data);
  }

  Map<String, dynamic> toJson() {
    return {
      'callbackUrl': this.callbackUrl,
      'purpose': this.purpose,
      'data': this.data,
    };
  }

  @override
  List<Object> get props => [
        callbackUrl,
        purpose,
        data,
      ];

  @override
  String toString() =>
      'M4eTransactionMeta(callbackUrl: $callbackUrl, purpose: $purpose, data: $data)';
}

/// Form parameters required for m4e-sdk transactions
///
class M4eTransactionForm extends Equatable {
  final M4eWallet from;
  final M4eWallet to;
  final M4eAmount amount;
  final DateTime at;
  final M4eTransactionMeta meta;
  final M4eTransactionStatus status;

  const M4eTransactionForm._(
      this.from, this.to, this.amount, this.at, this.meta, this.status);

  factory M4eTransactionForm({
    @required M4eWallet from,
    @required M4eWallet to,
    @required M4eAmount amount,
    @required DateTime at,
    @required M4eTransactionMeta meta,
    M4eTransactionStatus status = M4eTransactionStatus.pending,
  }) {
    assert(from != null, 'M4eTransactionForm [from] wallet should not be null');
    assert(to != null, 'M4eTransactionForm [to] wallet should not be null');
    assert(from != to,
        'M4eTransactionForm [from] and [to] wallets should not be the same');
    assert(amount != null, 'M4eTransactionForm [amount] should not be null');
    assert(at != null, 'M4eTransactionForm [at] should not be null');
    assert(meta != null, 'M4eTransactionForm [meta] should not be null');
    return M4eTransactionForm._(from, to, amount, at, meta, status);
  }

  Map<String, dynamic> toJson() {
    final flow = {
      'from': this.from.toJson(),
      'to': this.to.toJson(),
      'amount': this.amount.toMap(),
      'at': DateTime.now().toTimestampSinceEpoch()
      // TODO: add meta to object
    };

    return {'flow': flow, 'txType': 'CASH_IN'};
  }

  @override
  List<Object> get props => [
        from,
        to,
        amount,
        at,
        meta,
        status,
      ];

  @override
  String toString() {
    return 'M4eTransactionForm(from: $from, to: $to, amount: $amount, at: $at, meta: $meta, status: $status)';
  }
}
