import 'package:equatable/equatable.dart';
import 'package:flutter/foundation.dart';

import '../core/core.dart';
import 'models.dart';

class M4eTransactionLog extends Equatable {
  final M4eWallet from;
  final M4eWallet to;
  final M4eAmount amount;
  final DateTime at;
  final M4eTransactionType type;

  const M4eTransactionLog({
    @required this.from,
    @required this.to,
    @required this.amount,
    @required this.at,
    @required this.type,
  });

  factory M4eTransactionLog.fromJson(Map<String, dynamic> json) {
    return M4eTransactionLog(
      from: M4eWallet.fromJson(json['flow']['from']),
      to: M4eWallet.fromJson(json['flow']['to']),
      amount: M4eAmount.fromJson(json['flow']['amount']),
      at: DateTimeConverter.fromTimestampSinceEpoch(json['flow']['at']),
      type: M4eTransactionType.parse(json['txType']),
    );
  }

  Map<String, dynamic> toMap() {
    final flow = {
      'from': this.from.toJson(),
      'to': this.to.toJson(),
      'amount': this.amount.toMap(),
      'at': this.at.toTimestampSinceEpoch(),
    };

    return {'flow': flow, 'txType': this.type};
  }

  @override
  List<Object> get props => [
        from,
        to,
        amount,
        at,
        type,
      ];

  @override
  String toString() {
    return 'M4eTransactionLog(from: $from, to: $to, amount: $amount, at: $at, type: $type)';
  }
}

class M4eTransactionLogList extends Equatable {
  final List<M4eTransactionLog> transactionLogs;

  const M4eTransactionLogList({@required this.transactionLogs});

  factory M4eTransactionLogList.fromJson(Map<String, dynamic> json) {
    return M4eTransactionLogList(
      transactionLogs:
          (json['data'] as List)?.map((e) => e['transaction'])?.toList() ?? [],
    );
  }

  Map<String, dynamic> toMap() {
    return {
      'data': this
              .transactionLogs
              ?.map((e) => {'transaction': e.toMap()})
              ?.toList() ??
          []
    };
  }

  @override
  List<Object> get props => [transactionLogs];

  @override
  String toString() =>
      'M4eTransactionLogList(transactionLogs: $transactionLogs)';
}
