import 'package:equatable/equatable.dart';
import 'package:flutter/foundation.dart';

import '../core/core.dart';
import '../models/models.dart';

class M4eWalletInfo extends Equatable {
  final String label;
  final String currency;
  final M4eWalletType type;

  const M4eWalletInfo._(this.label, this.currency, this.type);

  factory M4eWalletInfo({
    @required String label,
    @required String currency,
    M4eWalletType type = M4eWalletType.bascule,
  }) {
    assert(label != null, 'WalletInfo [label] should not be null');
    assert(currency != null, 'WalletInfo [currency] should not be null');
    assert(type != null, 'WalletInfo [type] should not be null');
    return M4eWalletInfo._(label, currency, type);
  }

  factory M4eWalletInfo.fromJson(Map<String, dynamic> json) {
    return M4eWalletInfo(
      label: json['label'],
      currency: json['currency'],
      type: M4eWalletType.parse(json['type']),
    );
  }

  @override
  List<Object> get props => [
        label,
        currency,
        type,
      ];

  @override
  String toString() =>
      'M4eWalletInfo(label: $label, currency: $currency, type: $type)';
}

class M4eWallet extends Equatable {
  final M4eUniqueId id;
  final M4eWalletInfo walletInfo;

  const M4eWallet._(this.id, this.walletInfo);

  factory M4eWallet(
      {@required M4eUniqueId id, @required M4eWalletInfo walletInfo}) {
    assert(id != null, 'Wallet [id] should not be null');
    assert(walletInfo != null, 'Wallet [walletInfo] should not be null');
    return M4eWallet._(id, walletInfo);
  }

  factory M4eWallet.fromJson(Map<String, dynamic> json) {
    return M4eWallet(
      id: json['id'],
      walletInfo: M4eWalletInfo(
        label: json['wallet']['label'],
        currency: json['wallet']['currency'],
        type: M4eWalletType.parse(json['wallet']['type']),
      ),
    );
  }

  Map<String, dynamic> toJson() {
    return {
      'id': this.id.toString(),
      'wallet': {
        'label': this.walletInfo.label,
        'currency': this.walletInfo.currency,
        'type': this.walletInfo.type.toString()
      }
    };
  }

  @override
  List<Object> get props => [id, walletInfo];

  @override
  String toString() =>
      'M4eWallet(id: ${id.value.toString()}, walletInfo: ${walletInfo.toString()})';
}

class M4eWalletList extends Equatable {
  final List<M4eWallet> wallets;

  const M4eWalletList({@required this.wallets});

  factory M4eWalletList.fromJson(Map<String, dynamic> json) {
    final _data = json['data'] as List;

    return M4eWalletList(
        wallets: _data?.map((e) => M4eWallet.fromJson(e))?.toList() ?? []);
  }

  Map<String, dynamic> toJson() {
    return {
      'data': this.wallets?.map((e) => e.toJson())?.toList() ?? [],
    };
  }

  @override
  List<Object> get props => [wallets];

  @override
  String toString() => 'M4eWalletList(wallets: $wallets)';
}
