import 'dart:convert';

import 'package:dio/dio.dart';
import 'package:flutter_secure_storage/flutter_secure_storage.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:m4e_sdk/m4e_sdk.dart';
import 'package:m4e_sdk/src/core/core.dart';
import 'package:m4e_sdk/src/data/api/apis.dart';
import 'package:mockito/mockito.dart';

class MockDio extends Mock implements Dio {}

class MockFlutterSecureStorage extends Mock implements FlutterSecureStorage {}

class MockM4eLoggingInterceptors extends Mock
    implements M4eLoggingInterceptors {}

void main() {
  AuthApiImpl _authApi;

  MockDio mockDio;
  MockFlutterSecureStorage mockFlutterSecureStorage;
  MockM4eLoggingInterceptors mockM4eLoggingInterceptors;

  setUp(() {
    mockDio = MockDio();
    mockFlutterSecureStorage = MockFlutterSecureStorage();
    mockM4eLoggingInterceptors = MockM4eLoggingInterceptors();

    _authApi = AuthApiImpl(
      client: mockDio,
      storage: mockFlutterSecureStorage,
      loggingInterceptor: mockM4eLoggingInterceptors,
    );
  });

  final _challengeHeader =
      'a1aca99c-d289-43cc-9bec-3ca8388d06a2:111111:${DateTime.now().toTimestampSinceEpoch()}';

  final _m4eAuthPhoneNumberForm = M4eAuthPhoneNumberForm(
    id: M4eUniqueId.fromUniqueString('a1aca99c-d289-43cc-9bec-3ca8388d06a2'),
    phoneNumber: M4ePhoneNumber(number: '+233123456789', country: 'GHANA'),
  );

  final _m4eAuthOtpVerificationForm =
      M4eAuthOTPVerificationForm(otpCode: '111111');

  test('should generate appropriate challenge authorization header', () {
    // act
    final _base64Encoded = _authApi.base64EncodedChallenge(
        _m4eAuthPhoneNumberForm, _m4eAuthOtpVerificationForm);

    // assert
    final _base64Decoded = base64.decode(_base64Encoded);
    final _decodedToString = utf8.decode(_base64Decoded);

    expect(_challengeHeader, equals(_decodedToString));
  });
}
